#ifndef SW_SQUIDWALL_H
#define SW_SQUIDWALL_H

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "params.h"

#define VERSION    "squidwall 0.5a"

/* system */
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/un.h>

#include <pthread.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <sched.h>
#include <regex.h>
#include <fcntl.h>
#include <errno.h>
#include <ctype.h>
#include <time.h>
#include <math.h>

/* libowfat */
#include "array.h"
#include "buffer.h"
#include "case.h"
#include "fmt.h"
#include "ip4.h"
#include "ip6.h"
#include "io.h"
#include "str.h"
#include "open.h"
#include "scan.h"
#include "socket.h"
#include "stralloc.h"
#include "textcode.h"
#include "cache.h"

/* squidwall */
#include "request.h"
#include "sig.h"
#include "log.h"
#include "env.h"
#include "list.h"
#include "memcache.h"
#include "strallocX.h"
#include "thread.h"
#include "check.h"
#include "read_acls.h"
#include "read_controls.h"

/* type of acl directories */
#define TYPE_ACL      0
#define TYPE_GLOBAL   1
#define TYPE_STRING   2
#define TYPE_REGEX    3
#define TYPE_PHRASE   4
#define TYPE_REDIRECT 5

/* hosts/ip's + users */
struct squidwall_acl {
	struct list_head list; /* the list with all elements */
	stralloc filename;     /* without the '?' prefix */

	int flags:6;
#define ACL_NONE      0        /* no flag is set currently */
#define ACL_ALLOW     1        /* the host/user isn't filtered @all */
#define ACL_FILTER    2        /* the host/user has filtered access */
#define ACL_DENY      4        /* the host/user can't use the proxy */
#define ACL_REDIR     8        /* redirect with '302:' prefix (ssl and stuff) */
#define ACL_MULTI    16        /* redirect with '#nr ...' - new redirector interface */
#define ACL_MEMHIT   32        /* doesn't need memcache_insert() */

	int todo; /* XXX: remove */
#define TODO_NONE      0
#define TODO_URL       1
#define TODO_CONT      2
#define TODO_VIRUS     4
#define TODO_DEFAULT   8-1

	/* url filter - banners (filenames) */
	int b_todo:6;
	stralloc bfilter;      /*  1 +b1 */
	stralloc bfilter_url;  /*  2 +b2 */
	stralloc bfilter_dom;  /*  4 +b3 */
	stralloc bfilter_md5;  /*  8 +b4 */
	stralloc bfilter_sha1; /* 16 +b5 */
	stralloc brfilter;     /* 32 +b6 */

	/* url filter - white lists (filenames) */
	int w_todo:6;
	stralloc wfilter;      /*  1 +w1 */
	stralloc wfilter_url;  /*  2 +w2 */
	stralloc wfilter_dom;  /*  4 +w3 */
	stralloc wfilter_md5;  /*  8 +w4 */
	stralloc wfilter_sha1; /* 16 +w5 */
	stralloc wrfilter;     /* 32 +w6 */

	/* url filter - black lists (filenames) */
	int u_todo:6;
	stralloc ufilter;      /*  1 +u1 */
	stralloc ufilter_url;  /*  2 +u2 */
	stralloc ufilter_dom;  /*  4 +u3 */
	stralloc ufilter_md5;  /*  8 +u4 */
	stralloc ufilter_sha1; /* 16 +u5 */
	stralloc urfilter;     /* 32 +u6 */

	/* content filter (filenames) */
	int c_todo:4;
	stralloc bcfilter;     /* 1 +a */
	stralloc cfilter;      /* 2 +c */
	stralloc crfilter;     /* 4 +C */
	stralloc pfilter;      /* 8 +p */
};

/* bfilter, ufilter, cfilter, ufilter_md5, ufilter_sha1, wfilter, ...  */
struct squidwall_string {
	struct list_head list;  /* the list with all elements */
	stralloc filename;      /* just the filename of the filters */
	stralloc strings;       /* all strings from filename */
};

/* pfilter */
struct squidwall_phrase {
	struct list_head list;  /* the list with all elements */
	stralloc filename;      /* just the filename of the phrase filters */
	stralloc strings;       /* all string:hitpoint pairs from filename */
	array points;           /* array of type 'unsigned int' (0..255) */
};

/* bcfilter, brfilter, urfilter, crfilter, wrfilter */
struct squidwall_regex {
	struct list_head list;  /* the list with all elements */
	stralloc filename;      /* just the filename of the regex filters */
	array regex;            /* array of type 'struct named_regex' */
};

struct named_regex {
	stralloc n;  /* the string, which defines the regex */
	regex_t r;   /* the precompiled regex */
	int cflags;
	int eflags;
};

#define isset(v,x)     (((v)&(x)) == (x))
#define set(v,x)       (v) |=  (x)
#define unset(v,x)     (v) &= ~(x)

/* dummy request for debugging / logging from the main thread */
extern request_t *r;

#endif
